@@ -21,32 +21,50 @@ with this program; if not, write to the Free Software Foundation, Inc.,
 
 # include "prerex.h"
 
-PRIVATE element *first_cut;		/* stack of cut nodes */
+PRIVATE element *first_cut;            /* stack of cut nodes */
 PRIVATE done *first_done;               /* stack of commands done */
 
 
-char deftext[LINE_LEN];		/* buffer for default input */
+char deftext[LINE_LEN];            /* buffer for default input */
+
+# ifdef HAVE_LIBREADLINE
+int
+set_deftext (void)            /* used by readline to output defaults */
+{
+  rl_insert_text (deftext);
+  deftext[0] = '\0';
+  return 0;
+}
+# else
 PRIVATE char prompt[LINE_LEN];
-PRIVATE char *
+PRIVATE char *prompt_n;
+
+char *
 prompt_f (void)                   /* used to output prompts */
 { return prompt; }
+# endif
 
 char *
 Readline ( char * str )
 /* Uses str as the prompt and deftext as the "default" */
 {
-  int count;
+# ifdef HAVE_LIBREADLINE
+return readline (str);
+# else
   char *line;
+  int line_n;
   prompt[0] = '\0';
-  append (prompt, NULL, str, sizeof (prompt));
+  prompt_n = prompt;
+  append (prompt, &prompt_n, str, sizeof (prompt));
   el_push (el, deftext);
-  line = (char *) el_gets (el, &count);
+  line = (char *) el_gets (el, &line_n);
   if (line == NULL) error("Readline fails.");
-  line[count-1] = '\0';    /* replace '\n' by '\0' */ 
+  line[line_n - 1] = '\0';    /* replace '\n' by '\0' */ 
+  deftext[0] = '\0';
   return line;
+# endif
 }
 
-
 PRIVATE char *command_line;
 PRIVATE char *clp;
 
@@ -81,12 +99,11 @@ skip_command (void)
 PRIVATE void
 execute_shell_command (void)
 {
-  fclose (tex_file);
-  restore_write_access ();
+  fflush (tex_file);
+  if (fclose (tex_file) == EOF) error (".tex file closure failed.");
   if (system (clp)) puts("System call failed.");
   tex_file = fopen (chartfilename, "r+");
   if (tex_file == NULL) error ("Can't re-open the .tex file.");
-  remove_write_access ();
   close_files(); 
   free_lists();
   analyze_tex_file ();
@@ -139,7 +156,6 @@ shift (void)
     if (sscanf (clp, "%i", &p0.x) != 1) {
       break;
     }
-    if (pb == NULL)  /* first coordinate pair */
     while (isspace (*clp)) clp++;
     if (*clp == '-' || *clp == '+') clp++;
     while (isdigit(*clp)) clp++;
@@ -374,13 +390,13 @@ Raise (void)  /* "Raise" to avoid conflicting with signal.h function raise */
 PRIVATE void
 cut_node (point p)
 {
-  element *pn, *pnt;
+  element *pn, **ppn;
   done *this;
-  pnt = NULL;
+  ppn = &first_node;  /* used if the first node is removed from the list */
   pn = first_node;
   while (pn != NULL && !eq (p, pn->u.n.at))
     {
-      pnt = pn;
+      ppn = &(pn->next);
       pn = pn->next;
     }
   if (pn == NULL)
@@ -388,11 +404,8 @@ cut_node (point p)
       puts ("No course box, mini or text at this location.");
       return;
     }
+  *ppn = pn->next;  /* removes pn node from the list */
   /* move node to cut list */
-  if (pnt == NULL)
-    first_node = pn->next;
-  else
-    pnt->next = pn->next;
   pn->next = first_cut;
   first_cut = pn;
   this = (done *) malloc (sizeof (done)); 
@@ -405,13 +418,13 @@ cut_node (point p)
 PRIVATE void
 delete_node (point p)
 {
-  element *pn, *pnt;
+  element *pn, **ppn;
   done *this;
-  pnt = NULL;
+  ppn = &first_node;
   pn = first_node;
   while (pn != NULL && !eq (p, pn->u.n.at))
     {
-      pnt = pn;
+      ppn = &(pn->next);
       pn = pn->next;
     }
   if (pn == NULL)
@@ -419,10 +432,7 @@ delete_node (point p)
       puts ("No course box, mini or text at this location.");
       return;
     }
-  if (pnt == NULL)
-    first_node = pn->next;
-  else
-    pnt->next = pn->next;
+  *ppn = pn->next;
   this = (done *) malloc (sizeof (done)); 
   if (this == NULL) error ("Out of memory");
   this->tag = NODE_DELETE;
@@ -439,7 +449,7 @@ delete_arrow (point p0, point p1)
   element *n0 = node_at (p0);
   element *n1 = node_at (p1);
   element *pa;
-  element *pt; 
+  element **ppa; 
   if (n0 == NULL)
     {
       puts ("No course box, mini or text at the source location.");
@@ -451,10 +461,10 @@ delete_arrow (point p0, point p1)
       return;
     }
   pa = first_arrow;
-  pt = NULL;		/* trailing pointer */
+  ppa = &first_arrow; /* used if the first arrow is removed */          
   while (pa != NULL && (pa->u.a.source != n0 || pa->u.a.target != n1))
     {
-      pt = pa;
+      ppa = &(pa->next);
       pa = pa->next;
     }
   if (pa == NULL)
@@ -462,10 +472,7 @@ delete_arrow (point p0, point p1)
       puts ("No such arrow.");
       return;
     }
-  if (pt == NULL)
-    first_arrow = pa->next;
-  else
-    pt->next = pa->next;
+  *ppa = pa->next; /* removes pa arrow from the list */
   this = (done *) malloc (sizeof (done)); 
   if (this == NULL) error ("Out of memory");
   this->tag = ARROW_DELETE;
@@ -568,17 +575,14 @@ analyze_delete_command (void)
     }
     /* delete all nodes in box p0:p1 */
     {
-      element *pn, *pnt;
-      pnt = NULL;
+      element *pn, **ppn;
+      ppn = &(first_node);
       pn = first_node;
       while (pn != NULL) {
         if (inrange(pn->u.n.at.x, p0.x, p1.x) && inrange(pn->u.n.at.y, p0.y, p1.y)) 
         /* delete this node */
         {
-          if (pnt == NULL)
-            first_node = pn->next;
-          else
-            pnt->next = pn->next;
+          *ppn = pn->next;
           printf("Node at %i,%i deleted.\n", pn->u.n.at.x, pn->u.n.at.y);
           this = (done *) malloc (sizeof (done)); 
           if (this == NULL) error ("Out of memory");
@@ -588,7 +592,7 @@ analyze_delete_command (void)
           first_done = this;
           n_deletions++;
         }
-        else  pnt = pn;
+        else  ppn = &(pn->next);
         pn = pn->next;
       }
     }
@@ -617,34 +621,34 @@ undo (void)
       element *pn = this->e;
       if (insert_node (pn))
         {
-	  puts ("There is now another course box, mini or text at that location.");
+        puts ("There is now another course box, mini or text at that location.");
           return;
         }
       printf ("Course box, mini or text at %i,%i restored.\n",
-	      pn->u.n.at.x, pn->u.n.at.y);
+            pn->u.n.at.x, pn->u.n.at.y);
     } 
   else if (this->tag == ARROW_DELETE)
     { 
       element *pn = this->e;
       if (insert_arrow (pn))
-	{
-	  puts
-	    ("Can't undelete arrow: source and/or target is/are now missing.");
-	  return;
-	}
+      {
+        puts
+          ("Can't undelete arrow: source and/or target is/are now missing.");
+        return;
+      }
       printf ("Arrow from %i,%i to %i,%i restored.\n",
-	      pn->u.a.source->u.n.at.x, pn->u.a.source->u.n.at.y,
-	      pn->u.a.target->u.n.at.x, pn->u.a.target->u.n.at.y);
+            pn->u.a.source->u.n.at.x, pn->u.a.source->u.n.at.y,
+            pn->u.a.target->u.n.at.x, pn->u.a.target->u.n.at.y);
     }
   else if (this->tag == NODE_CREATE)
     {
-      element *pn, *pnt;
+      element *pn, **ppn;
       point p = this->p;
-      pnt = NULL;
+      ppn = &(first_node);
       pn = first_node;
       while (pn != NULL && !eq (p, pn->u.n.at))
         {
-          pnt = pn;
+          ppn = &(pn->next);
           pn = pn->next;
         }
       if (pn == NULL)
@@ -652,22 +656,19 @@ undo (void)
           puts ("No course box, mini or text at this location.");
           return;
         }
-      if (pnt == NULL)
-        first_node = pn->next;
-      else
-        pnt->next = pn->next;
+      *ppn = pn->next;
       printf("Course box, mini or text at %i,%i deleted.\n", p.x, p.y);
     }
   else if (this->tag == ARROW_CREATE)
     {
-      element *n0, *n1, *pa, *pt;
+      element *n0, *n1, *pa, **ppa;
       n0 = this->a.source;
       n1 = this->a.target;
       pa = first_arrow;
-      pt = NULL;		/* trailing pointer */
+      ppa = &(first_arrow); 
       while (pa != NULL && (pa->u.a.source != n0 || pa->u.a.target != n1))
         {
-          pt = pa;
+          ppa = &(pa->next);
           pa = pa->next;
         }
       if (pa == NULL)
@@ -675,10 +676,7 @@ undo (void)
           printf("There is no arrow from %i,%i to %i,%i.\n", n0->u.n.at.x, n0->u.n.at.y, n1->u.n.at.x, n1->u.n.at.y);
           return;
         }
-      if (pt == NULL)
-        first_arrow = pa->next;
-      else
-        pt->next = pa->next;
+      *ppa = pa->next;
       printf("Arrow from %i,%i to %i,%i deleted.\n", n0->u.n.at.x, n0->u.n.at.y, n1->u.n.at.x, n1->u.n.at.y);
     }
   else if (this->tag == NODE_EDIT)
@@ -728,13 +726,13 @@ undo (void)
     }
   else if (this->tag == PASTE)
     {
-      element *pn, *pnt;
+      element *pn, **ppn;
       element *pe = this->e;
-      pnt = NULL;
+      ppn = &(first_node);
       pn = first_node;
       while (pn != NULL && pn != pe)
         {
-          pnt = pn;
+          ppn = &(pn->next);
           pn = pn->next;
         }
       if (pn == NULL)
@@ -742,10 +740,7 @@ undo (void)
           puts ("Unable to find pasted node.");
           return;
         }
-      if (pnt == NULL)
-        first_node = pn->next;
-      else
-        pnt->next = pn->next;
+      *ppn = pn->next;
       pn->next = first_cut;
       printf ("Course box, mini or text unpasted from %i,%i.\n", pn->u.n.at.x, pn->u.n.at.y);
       pn->u.n.at = this->p;
@@ -761,7 +756,6 @@ undo (void)
 PRIVATE void
 edit_mini (element * pm)
 {
-  deftext[0] = '\0';
   append (deftext, NULL, pm->u.n.code, sizeof (deftext));
   command_line = Readline ("course code: ");
   pm->u.n.code[0] = '\0';
@@ -773,7 +767,6 @@ edit_mini (element * pm)
 PRIVATE void
 edit_text (element * pm)
 {
-  deftext[0] = '\0';
   append (deftext, NULL, pm->u.n.u.t.txt, sizeof (deftext));
   command_line = Readline ("text: ");
   pm->u.n.u.t.txt[0] = '\0';
@@ -786,14 +779,12 @@ PRIVATE void
 edit_box (element * pb)
 {
   char code[8];
-  deftext[0] = '\0';
   append (deftext, NULL, pb->u.n.code, sizeof (deftext));
   command_line = Readline ("course code: ");
   pb->u.n.code[0] = '\0';
   if (append (pb->u.n.code, NULL, command_line, sizeof (pb->u.n.code))
       >= sizeof (pb->u.n.code))
     puts ("Warning: course code too long, truncated.");
-  deftext[0] = '\0';
   if (pb->u.n.u.b.half)
     append (deftext, NULL, "y", sizeof (deftext));
   else
@@ -809,7 +800,6 @@ edit_box (element * pb)
       puts ("Response not recognized; 'y' assumed.");
       pb->u.n.u.b.half = true;
     }
-  deftext[0] = '\0';
   if (pb->u.n.u.b.req_opt == REQ)
     append (deftext, NULL, "r", sizeof (deftext));
   else if (pb->u.n.u.b.req_opt == OPT)
@@ -829,14 +819,12 @@ edit_box (element * pb)
       puts ("Response not recognized; 'n' assumed.");
       pb->u.n.u.b.req_opt = NEITHER;
     }
-  deftext[0] = '\0';
   append (deftext, NULL, pb->u.n.u.b.title, sizeof (deftext));
   command_line = Readline ("course title: ");
   pb->u.n.u.b.title[0] = '\0';
   if (append (pb->u.n.u.b.title, NULL, command_line, sizeof (pb->u.n.u.b.title))
       >= sizeof (pb->u.n.u.b.title))
     puts ("Warning: course title too long, truncated.");
-  deftext[0] = '\0';
   append (deftext, NULL, pb->u.n.u.b.timetable, sizeof (deftext));
   command_line = Readline ("course timetable: ");
   pb->u.n.u.b.timetable[0] = '\0';
@@ -844,7 +832,6 @@ edit_box (element * pb)
       (pb->u.n.u.b.timetable, NULL, command_line,
        sizeof (pb->u.n.u.b.timetable)) >= sizeof (pb->u.n.u.b.timetable))
     puts ("Warning: course timetable too long, truncated.");
-  deftext[0] = '\0';
   append (deftext, NULL, pb->u.n.u.b.color, sizeof (deftext));
   command_line = Readline ("background color (empty string for default): ");
   pb->u.n.u.b.color[0] = '\0';
@@ -859,7 +846,6 @@ PRIVATE void
 set_curvature (element * pa)
 {
   char code[16];
-  deftext[0] = '\0';
   if (pa->u.a.curvature >= 0 && pa->u.a.curvature <= 100)
     {
       sprintf (deftext, "%i", pa->u.a.curvature);
@@ -868,7 +854,7 @@ set_curvature (element * pa)
     {
       append (deftext, NULL, "d", sizeof (deftext));
     }
-    command_line = Readline ("curvature, default (d) or int value? ");
+  command_line = Readline ("curvature, default (d) or int value? ");
   sscanf (command_line, "%15s", code);
   if (isdigit (code[0]))
     {
@@ -879,7 +865,7 @@ set_curvature (element * pa)
       }
     }
   else if (code[0] == 'd')
-    pa->u.a.curvature = -1;	/* negative value denotes default curvature */
+    pa->u.a.curvature = -1;      /* negative value denotes default curvature */
   else
     {
       puts ("Response not recognized; default assumed.");
@@ -891,7 +877,6 @@ PRIVATE void
 edit_arrow (element * pa)
 {
   char code[16];
-  deftext[0] = '\0';
   switch (pa->u.a.tag)
     {
     case PREREQ:
@@ -1057,7 +1042,6 @@ analyze_box_command (void)
       return;
     }
   this->tag = NODE_CREATE;
-  deftext[0] = '\0';
   append (deftext, NULL, "y", sizeof (deftext));
   command_line = Readline ("create new course box (y/n)? ");
   sscanf (command_line, "%7s", code);
@@ -1074,12 +1058,12 @@ analyze_box_command (void)
       pb->u.n.u.b.title[0] = '\0';
       pb->u.n.u.b.timetable[0] = '\0';
       if (insert_node (pb))
-	{
-	  puts ("There is already a course box, mini, or text at that location.");
+      {
+        puts ("There is already a course box, mini, or text at that location.");
           free (pb);
           free (this);
-	  return;
-	}
+        return;
+      }
       this->next = first_done;
       this->p = p; /* record coordinates of new node */
       first_done = this;
@@ -1118,13 +1102,13 @@ analyze_mini_command (void)
       this->tag = NODE_EDIT;
       this->next = first_done;
       this->n = pm->u.n;  /* record the old node */
+      this->e = pm;
       first_done = this;
       edit_mini (pm);
       regenerate_and_process ();
       return;
     }
   this->tag = NODE_CREATE;
-  deftext[0] = '\0';
   append (deftext, NULL, "y", sizeof (deftext));
   command_line = Readline ("create new mini (y/n)? ");
   sscanf (command_line, "%7s", code);
@@ -1137,12 +1121,12 @@ analyze_mini_command (void)
       pm->u.n.at = p;
       pm->u.n.code[0] = '\0';
       if (insert_node (pm))
-	{
-	  puts ("There is already a course box, mini, or text at that location.");
+      {
+        puts ("There is already a course box, mini, or text at that location.");
           free (pm);
           free (this);
-	  return;
-	}
+        return;
+      }
       this->next = first_done;
       this->p = p; /* record coordinates of new node */
       first_done = this;
@@ -1180,13 +1164,13 @@ analyze_text_command (void)
       this->tag = NODE_EDIT;
       this->next = first_done;
       this->n = pm->u.n;  /* record the old node */
+      this->e = pm;
       first_done = this;
       edit_text (pm);
       regenerate_and_process ();
       return;
     }
   this->tag = NODE_CREATE;
-  deftext[0] = '\0';
   append (deftext, NULL, "y", sizeof (deftext));
   command_line = Readline ("create new text (y/n)? ");
   sscanf (command_line, "%7s", code);
@@ -1199,12 +1183,12 @@ analyze_text_command (void)
       pm->u.n.at = p;
       pm->u.n.code[0] = '\0';
       if (insert_node (pm))
-	{
-	  puts ("There is already a course box, mini, or text at that location.");
+      {
+        puts ("There is already a course box, mini, or text at that location.");
           free (pm);
           free (this);
-	  return;
-	}
+        return;
+      }
       this->next = first_done;
       this->p = p; /* record coordinates of new node */
       first_done = this;
@@ -1266,35 +1250,34 @@ analyze_arrow_command (void)
     {
       char code[8];
       this->tag = ARROW_CREATE;
-      deftext[0] = '\0';
       append (deftext, NULL, "y", sizeof (deftext));
       command_line = Readline ("create new arrow (y/n)? ");
       sscanf (command_line, "%7s", code);
       if (code[0] == 'y')
-	{
-	  pa = (element *) malloc (sizeof (element));
-	  if (pa == NULL) error ("Out of memory");
-	  pa->tag = ARROW;
-	  pa->u.a.source = n0;
-	  pa->u.a.target = n1;
-	  pa->u.a.tag = PREREQ;
-	  pa->u.a.curvature = -1;	/* default curvature */
-	  insert_arrow (pa);
+      {
+        pa = (element *) malloc (sizeof (element));
+        if (pa == NULL) error ("Out of memory");
+        pa->tag = ARROW;
+        pa->u.a.source = n0;
+        pa->u.a.target = n1;
+        pa->u.a.tag = PREREQ;
+        pa->u.a.curvature = -1;      /* default curvature */
+        insert_arrow (pa);
           this->next = first_done;
           this->a = pa->u.a; /* record arrow */
           first_done = this;
-	  edit_arrow (pa);
+        edit_arrow (pa);
           regenerate_and_process ();
-	}
+      }
       else if (code[0] == 'n')
         {
-	  free (this);
+        free (this);
         }
       else
-	{
-	  puts ("Response not recognized; 'n' assumed.");
-	  free (this);
-	}
+      {
+        puts ("Response not recognized; 'n' assumed.");
+        free (this);
+      }
     }
 }
 
@@ -1302,14 +1285,13 @@ PRIVATE void
 analyze_grid_command (void)
 {
   char code[8];
-  deftext[0] = '\0';
   if (sscanf (command_line, "%*s %7s", code) != 1)
     {
       if (grid)
-	append (deftext, NULL, "n", sizeof (deftext));
+           append (deftext, NULL, "n", sizeof (deftext));
       else
-	append (deftext, NULL, "y", sizeof (deftext));
-        command_line = Readline ("grid (y/n)? ");
+           append (deftext, NULL, "y", sizeof (deftext));
+      command_line = Readline ("grid (y/n)? ");
       sscanf (command_line, "%7s", code);
     }
   if (code[0] == 'y')
@@ -1334,7 +1316,6 @@ PRIVATE void
 backup (void)
 {
   char code[8];
-  deftext[0] = '\0';
   append (deftext, NULL, "y", sizeof (deftext));
   command_line = Readline ("Overwrite current backup file (y/n)? ");
   sscanf (command_line, "%7s", code);
@@ -1361,17 +1342,17 @@ restore (void)
       puts ("Can't open backup file.");
       return;
     }
-  deftext[0] = '\0';
   append (deftext, NULL, "y", sizeof (deftext));
-  command_line = Readline ("Delete current TeX file and restore from backup (y/n)? ");
+  command_line =
+    Readline ("Delete current TeX file and restore from backup (y/n)? ");
   sscanf (command_line, "%7s", code);
   if (code[0] != 'y')
     return;
-  restore_write_access ();
+  fclose (tex_file);
   tex_file = fopen (chartfilename, "w+");
   rewind (tex_file); 
   copy (backup_tex_file, tex_file);
-  remove_write_access ();
+  fflush (tex_file);
   fclose (backup_tex_file);
   close_files();
   free_lists();
@@ -1385,19 +1366,22 @@ PRIVATE void
 quit (void)
 {
   char code[8];
-  deftext[0] = '\0';
   if (first_cut)
     {
       append (deftext, NULL, "y", sizeof (deftext));
-      command_line = Readline ("Warning: there are unpasted cuts; continue (y/n)? ");
+      command_line =
+        Readline ("Warning: there are unpasted cuts; continue (y/n)? ");
       sscanf (command_line, "%7s", code);
       if (code[0] != 'y')
         return;
     }
+# ifdef HAVE_LIBEDITLINE
+  history_end (hist);
+  el_end (el);
+# endif
   grid = false;
   puts("Turning off coordinate grid.");
   regenerate_and_process ();
-  restore_write_access ();
   exit (0);
 }
 
@@ -1468,7 +1452,7 @@ analyze_user_command (void)
   else if (prefix (command, "arrow"))
     analyze_arrow_command ();
   else if (prefix (command, "quit")
-	   || prefix (command, "exit") || prefix (command, "x"))
+         || prefix (command, "exit") || prefix (command, "x"))
     {
       quit ();
     }
@@ -1480,19 +1464,19 @@ analyze_user_command (void)
     {
       clp = command_line;
       while (isspace (*clp))
-	clp++;
-      clp++;			/* ! */
+      clp++;
+      clp++;                  /* ! */
       while (isspace (*clp))
-	clp++;
+      clp++;
       if (*clp == '\0')
-	{
+      {
           reprocess = true;
-	  regenerate_and_process ();
-	}
+        regenerate_and_process ();
+      }
       else
-	{
-	  execute_shell_command ();
-	}
+      {
+        execute_shell_command ();
+      }
     }
   else if (prefix (command, "Backup"))
     backup ();
@@ -1511,42 +1495,58 @@ analyze_user_command (void)
 
 void process_commands(void)
 {
-  History *hist;
-  HistEvent ev;
+# ifdef HAVE_LIBREADLINE
+  char prompt[FILE_LEN + 8] = {'\0'};
   char *prompt_n = prompt;
-  help ();
-  hist = history_init();
-  history(hist, &ev, H_SETSIZE, 800);
-  el_set (el, EL_HIST, history, hist);
+  rl_startup_hook = set_deftext;
+  append (prompt, &prompt_n, chartfilename, sizeof (prompt));
+  prompt_n = prompt_n - 4; *prompt_n = '\0'; /* suppress ".tex"  */
+  append (prompt, &prompt_n, "> ", sizeof (prompt));
+# else
   el_set (el, EL_PROMPT, &prompt_f);
-  el_set (el, EL_EDITOR, "emacs");
+  deftext[0] = '\0';
+# endif
+  help ();
   do
     {
-      int count;
+# ifdef HAVE_LIBREADLINE
+      command_line = Readline (prompt);
+# else
+      int command_line_n;  
       prompt[0] = '\0';
       prompt_n = prompt;
       append (prompt, &prompt_n, chartfilename, sizeof (prompt));
       prompt_n = prompt_n - 4; *prompt_n = '\0'; /* suppress ".tex"  */
       append (prompt, &prompt_n, "> ", sizeof (prompt));
-      command_line = (char *) el_gets (el, &count);
+      command_line = (char *) el_gets (el, &command_line_n);
+# endif
       if (command_line == NULL)    /* EOF */
-      { putchar ('\n');
-	quit();		
-      }
+      { putchar ('\n'); quit(); }
+# ifdef HAVE_LIBREADLINE
+      if (command_line[0] != '\0')
+# else
       if (command_line[0] != '\n')
-	{
-	  history(hist, &ev, H_ENTER, command_line);
-          count--;
-          command_line[count] = '\0';
-          reprocess = true;
-          if (suffix(";", command_line))
-          {
-            /* suppress possible regeneration and processing after this command */
+# endif
+      {
+# ifdef HAVE_LIBREADLINE
+        add_history (command_line);
+# else
+        history(hist, &ev, H_ENTER, command_line);
+        command_line_n--;
+        command_line[command_line_n] = '\0';
+# endif
+        reprocess = true;
+        if (suffix(";", command_line))
+        { /* suppress possible regeneration and processing after this command */
             reprocess = false;
-            command_line[count-1] = '\0';
+# ifdef HAVE_LIBREADLINE
+            command_line[strlen(command_line) - 1] = '\0';
+# else
+            command_line[command_line_n - 1] = '\0';
+# endif
           }
-	  analyze_user_command ();
-	}
+        analyze_user_command ();
+      }
      }
-  while (true);			/* exit via call to exit or error()  */
+  while (true);         /* exit via call to exit or error()  */
 }
