@@ -50,14 +50,14 @@ bool
 insert_node (element * pn)
 {
   element *p;
-  element *pt;  /* trailing pointer */
+  element **pp;  
 
   /* ordered list */
   p = first_node;
-  pt = NULL;
+  pp = &(first_node);
   while ((p != NULL) && gt (p->u.n.at, pn->u.n.at))
     {
-      pt = p;
+      pp = &(p->next);
       p = p->next;
     }
   if ((p != NULL) && eq (p->u.n.at, pn->u.n.at))
@@ -65,10 +65,7 @@ insert_node (element * pn)
       /* node at that point already */
       return true;
     }
-  if (pt == NULL)
-    first_node = pn;
-  else
-    pt->next = pn;
+  *pp = pn;
   pn->next = p;
   return false;
 }  /* insert_node */
@@ -778,12 +775,12 @@ merge_lists (void)
      Then restore the saved arrows.
    */
   element *fn = first_node;
-  element *fa, *fat;		/* fat is a trailing pointer */
+  element *fa, **ffa;		
   element *save_arrow = NULL;	/* arrows already output  */
   while (fn != NULL)
     {
       output_node (fn);
-      fat = NULL;
+      ffa = &first_arrow;
       fa = first_arrow;
       while (fa != NULL)
 	{
@@ -798,26 +795,20 @@ merge_lists (void)
 	      if (ffn == fn)
 		{
 		  /* source node not yet output; defer arrow output */
-		  fat = fa;
+		  ffa = &(fa->next);
 		  fa = fa->next;
 		  break;
 		}
 	      output_arrow (fa);
 	      /* move arrow to save_arrow list: */
-	      if (fat == NULL)
-		first_arrow = fa->next;
-	      else
-		fat->next = fa->next;
+	      *ffa = fa->next;
 	      fa->next = save_arrow;
 	      save_arrow = fa;
-	      if (fat == NULL)
-		fa = first_arrow;
-	      else
-		fa = fat->next;
+	      fa = *ffa;
 	    }
 	  else
 	    {
-	      fat = fa;
+	      ffa = &(fa->next);
 	      fa = fa->next;
 	    }
 	}
@@ -868,7 +859,7 @@ void
 regenerate_tex_file (void)  /* internal representation -> tex file  */
 {
   printf ("Saving to %s.\n", chartfilename);
-  restore_write_access ();
+  fclose (tex_file);
   tex_file = fopen (chartfilename, "w+");
   copy (pretext, tex_file);
   if (grid)
@@ -878,6 +869,5 @@ regenerate_tex_file (void)  /* internal representation -> tex file  */
   copy (posttext, tex_file);
   fflush(tex_file);
   rewind (tex_file);
-  remove_write_access ();
 }
 
